package cn.com.ty.lift.push.websocket.handler;

import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.common.constants.WebSocketConstants;
import cn.com.ty.lift.common.model.TimeyMessage;
import cn.com.ty.lift.push.websocket.dao.model.RealTimeMessage;
import cn.com.ty.lift.push.websocket.service.TimeyMessageService;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.web.socket.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class TimeyMessageHandler implements WebSocketHandler {

    private TimeyMessageService timeyMessageService;

    public TimeyMessageHandler() {
    }

    public TimeyMessageHandler(TimeyMessageService timeyMessageService) {
        this.timeyMessageService = timeyMessageService;
    }

    //在线用户列表
    private static final Map<Long, WebSocketSession> users;

    static {
        users = new HashMap<>();
    }

    //新增socket
    @Override
    public void afterConnectionEstablished(WebSocketSession session) {
        //用户登录连接后，将用户id放入到在线用户列表中
        Long userId = getCurrentUser(session);
        users.put(userId, session);
        //获取用户实时消息
        List<TimeyMessage> timeyMessageList = timeyMessageService.list(new QueryWrapper<TimeyMessage>()
                .eq("view_flag", WebSocketConstants.IS_NOT_VIEW)
                .eq("user_id", userId)
        );
        //将消息推送给用户
        pushMessageToUser(userId, timeyMessageList);
    }

    //接收socket信息
    @Override
    public void handleMessage(WebSocketSession webSocketSession, WebSocketMessage<?> webSocketMessage) {

    }

    /**
     * @param timeyMessages 推送的消息
     * @return 是否推送成功
     * @description 保存用户信息，并将消息推送给多个用户
     * @date 2020/5/2 4:28 下午
     */
    public boolean saveAndPushMessageToUsers(List<TimeyMessage> timeyMessages) {
        if (timeyMessages != null && timeyMessages.size() > 0) {
            //先将即使消息保存到数据库
            if (timeyMessageService.saveBatch(timeyMessages)) {
                List<Long> userIds = new ArrayList<>();
                timeyMessages.forEach(timeyMessage -> {
                    userIds.add(timeyMessage.getUserId());
                });
                return pushMessageToUsers(userIds, timeyMessages);
            }
        }
        return false;
    }

    /**
     * @return boolean
     * @Author huangyuan
     * @Description 给多个用户推送消息
     * @Date 21:36 2019-04-19
     * @Param [userIds, timeyMessages, count]
     **/
    public static boolean pushMessageToUsers(List<Long> userIds, List<TimeyMessage> timeyMessages) {
        try {
            if (userIds != null && userIds.size() > 0) {
                for (Long userId : userIds) {
                    WebSocketSession session = users.get(userId);
                    if (session != null) {
                        //新建消息体
                        RealTimeMessage realTimeMessage;
                        if (timeyMessages != null && timeyMessages.size() > 0) {
                            TimeyMessage timeyMessage = timeyMessages.get(0);
                            realTimeMessage = new RealTimeMessage(timeyMessages);
                            realTimeMessage.setSize((long) timeyMessages.size());
                            realTimeMessage.setMsgType(timeyMessage.getType());
                        } else {
                            realTimeMessage = new RealTimeMessage();
                            realTimeMessage.setSize(0L);
                        }
                        session.sendMessage(new TextMessage(JSONUtil.parseObj(realTimeMessage).toString()));
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }


    /**
     * @return boolean
     * @Author huangyuan
     * @Description 把消息推给单个用户
     * @Date 21:35 2019-04-19
     * @Param [userId, timeyMessages]
     **/
    public static boolean pushMessageToUser(Long userId, List<TimeyMessage> timeyMessages) {
        List<Long> userIds = new ArrayList<>();
        userIds.add(userId);
        return pushMessageToUsers(userIds, timeyMessages);
    }

    /**
     * @return boolean
     * @Author huangyuan
     * @Description 给单独用户推送信息
     * @Date 00:35 2019-04-20
     * @Param [userId, timeyMessage]
     **/
    public static boolean pushMessageToUser(Long userId, TimeyMessage timeyMessage) {
        List<TimeyMessage> timeyMessageList = new ArrayList<>();
        timeyMessageList.add(timeyMessage);
        return pushMessageToUser(userId, timeyMessageList);
    }

    @Override
    public void handleTransportError(WebSocketSession session, Throwable exception) throws Exception {
        if (session.isOpen()) {
            session.close();
        }
        System.out.println("连接出错");
        users.remove(getCurrentUser(session));
    }

    @Override
    public void afterConnectionClosed(WebSocketSession session, CloseStatus status) throws Exception {
        System.out.println("连接已关闭：" + status);
        users.remove(getCurrentUser(session));
    }

    @Override
    public boolean supportsPartialMessages() {
        return false;
    }

    /**
     * 获取用户id，作为标识
     *
     * @param session
     * @return Long
     * @throws
     * @author huangy
     * @date 2018年9月18日
     */
    private Long getCurrentUser(WebSocketSession session) {
        Long userId;
        try {
            userId = (Long) session.getAttributes().get(ApiConstants.CURRENT_USER_ID);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
        return userId;
    }

}
